# Cleanup R environment
rm(list=ls())               # Remove data frames and objects
dev.off()                   # Remove plots
cat("\014")                 # Remove results from the console

# Set output to be written with comma or dot for separating decimals
options(OutDec= ".")

# Set working directory
setwd("C:/Taiacupeba_SedimentModel")

# Load data sets
SedData <- read.csv("SedimentData_Balainho.csv",header=T)
  SedData$SAMPLE_DATE <- as.Date(SedData$SAMPLE_DATE, "%m/%d/%Y")

# Install Nonlinear Least Squares regression package for fist run
# install.packages("nlstools")

# Load nlstools
library("nlstools")
  
#Attach data set, as required by nlstools
attach(SedData)

# Assign a formula for the non-linear regression
formulaPwr <- as.formula(SSC ~ a*(Q^b))

# Find starting values for the coefficients
preview(formulaPwr, SedData, list(a = 185, b = 2))

# Once acceptable starting values were obtained, write them into the nls function
PwrFit1 <- nls(formulaPwr, SedData, list(a = 185, b = 2))

# Overview model parameters
overview(PwrFit1)

# Retrieve estimations
est <- nlsResiduals(PwrFit1)

# Get observed, simulated values and residuals
obs <- SSC
sim <- fitted(PwrFit1)
res <- est$resi1[,2]         # Un-transformed residuals
res.i <- est$resi4[,1]       # Residuals i for autocorrelation
res.i1 <- est$resi4[,2]      # Residuals i+1 for autocorrelation
std.res <- est$resi2[, 2]    # Standardized residuals

# Compute the Anderson Darling normality test. Alpha shows the significance level
# under which the hypothesis of normal distribution is to be accepted
library(nortest)
ad.test(res)

# Compute model efficiency with Nash-Sutcliffe efficiency index
NS <- 1-(sum((sim-obs)^2)/sum((obs-mean(obs))^2))
str(NS)

# Compute Kendall's Tau Correlation Coefficient
cor(obs, sim, method = "kendall")

#Outlier analysis using jackknife resampling procedure
OutLier1 <- nlsJack(PwrFit1)
summary(OutLier1)

#Jackknife parameter estimates
OutLier1$coefjack

#Absolute relative difference (in percent of the estimate) of the estimates with and without each observation
OutLier1$reldif

#Plot fitted model
par(mar=c(5,6,4,1))
plotfit(PwrFit1, smooth = TRUE,
        xlab = "Discharge, in cubic meters per second",
        ylab = "Suspended sediment concentration, \nin milligrams per liter",
        xaxt = "n", yaxt = "n", pch.obs = "O", cex = 1.5, lwd = 3, cex.lab = 1.5)
axis(1, at = pretty(Q, 5), labels = pretty(Q, 5), tck = 0.04, cex.axis = 1.5)
axis(2, at = pretty(SSC, 5), labels = pretty(SSC, 5), tck = 0.03, cex.axis = 1.5)
axis(3, at = pretty(Q, 5), labels = F, tck = 0.04, cex.axis = 1.5)
axis(4, at = pretty(SSC, 5), labels = F, tck = 0.03, cex.axis = 1.5)
legend("topleft", inset = 0.05, c("Fitted curve", "Sampled events"), lty = c(1, NA),
       lwd = c(3, NA), pch = c(NA, "O"), col = c("red", "black"), bty = "n", cex = 1.5,
       title = "Explanation", title.adj = 0.1)

# Residuals Plot
par(mar=c(5,6,4,16))
plot(sim, res, xlab = "Simulated values", ylab = "Residuals",
     xaxt = "n", yaxt = "n", pch = "x", cex = 1.5, cex.lab = 1.5)
axis(1, at = pretty(sim, 5), labels = pretty(sim, 5), tck = 0.04, cex.axis = 1.5)
axis(2, at = pretty(res, 5), labels = pretty(res, 5), tck = 0.03, cex.axis = 1.5)
axis(3, at = pretty(sim, 5), labels = F, tck = 0.04, cex.axis = 1.5)
axis(4, at = pretty(res, 5), labels = F, tck = 0.03, cex.axis = 1.5)
abline(h = 0, lty = 2, lwd = 2)

# Autocorrelation
plot(res.i, res.i1, xlab = "Residual i", ylab = "Residual i+1",
     xaxt = "n", yaxt = "n", pch = "x", cex = 1.5, cex.lab = 1.5)
axis(1, at = pretty(res.i, 5), labels = pretty(res.i, 5), tck = 0.04, cex.axis = 1.5)
axis(2, at = pretty(res.i1, 5), labels = pretty(res.i1, 5), tck = 0.03, cex.axis = 1.5)
axis(3, at = pretty(res.i, 5), labels = F, tck = 0.04, cex.axis = 1.5)
axis(4, at = pretty(res.i1, 5), labels = F, tck = 0.03, cex.axis = 1.5)
abline(h = 0, lty = 2, lwd = 2)

# Q-Q Plot of Standardized Residuals
qqnorm(std.res, main = "", xlab = "Theoretical quantiles", ylab = "Sample quantiles",
       xaxt = "n", yaxt = "n", pch = "O", cex = 1.5, cex.lab = 1.5)
axis(1, at = seq(-2, 2, 1), labels = seq(-2, 2, 1), tck = 0.04, cex.axis = 1.5)
axis(2, at = seq(-2, 2, 1), labels = seq(-2, 2, 1), tck = 0.03, cex.axis = 1.5)
axis(3, at = seq(-2, 2, 1), labels = F, tck = 0.04, cex.axis = 1.5)
axis(4, at = seq(-2, 2, 1), labels = F, tck = 0.03, cex.axis = 1.5)
qqline(std.res, lwd = 3)

# Residuals against time
plot(SAMPLE_DATE, res, xlab = "Time", ylab = "Residuals",
     xaxt = "n", yaxt = "n", pch = "x", cex = 1.5, cex.lab = 1.5)
axis.Date(1, SAMPLE_DATE, format = "%Y", tck = 0.04, cex.axis = 1.5)
axis(2, at = pretty(res, 5), labels = pretty(res, 5), tck = 0.03, cex.axis = 1.5)
axis.Date(3, SAMPLE_DATE, labels = F, tck = 0.04, cex.axis = 1.5)
axis(4, at = pretty(res, 5), labels = F, tck = 0.03, cex.axis = 1.5)
abline(h = 0, lty = 2, lwd = 2)

# Retrieve coefficients
coef <- summary(PwrFit1)
a <- coef$coefficients[1,1]
b <- coef$coefficients[1,2]

# Create a data frame with total suspended load and suspended load per grain class
# HECRAS.data is ready for copy/paste into HEC-RAS
HECRAS.data <- data.frame("Flow" = seq(0.094, 7.182, ((7.182-0.094)/10)))
HECRAS.data <- transform(HECRAS.data, "TotalLoad" = Flow * (a * (Flow ^ b)) * 0.0864)
HECRAS.data <- transform(HECRAS.data, "Clay" = TotalLoad * 0.103)
HECRAS.data <- transform(HECRAS.data, "VFM" = TotalLoad * 0.200)
HECRAS.data <- transform(HECRAS.data, "FM" = TotalLoad * 0.347)
HECRAS.data <- transform(HECRAS.data, "MM" = TotalLoad * 0.240)
HECRAS.data <- transform(HECRAS.data, "CM" = TotalLoad * 0.084)
HECRAS.data <- transform(HECRAS.data, "VFS" = TotalLoad * 0.026)
HECRAS.data <- t(HECRAS.data)

#Detach data set
detach(SedData)